import java.lang.reflect.Array;

public class ClosedTable<K, V> implements Table<K, V> {
	private Pair<K, V>[] contents;
	private int size;
	private Pair<K, V> removed = new Pair<K, V>();
	@SuppressWarnings("unchecked")
	public ClosedTable(int size) {
		this.size = size;
		Pair<K, V> pair = new Pair<K, V>();
		contents = (Pair<K, V>[])Array.newInstance(pair.getClass(), size);
		for(int i = 0; i < size; i++) {
			contents[i] = null;
		}
	}
	public V get(K k) {
		int start = k.hashCode() % size;
		for(int i = 0; i < size; i++) {
			int index = (i + start) % size;
			Pair<K, V> elem = contents[index];
			if (elem == null) return null;
			if (elem == removed) continue;
			if (elem.fst.equals(k)) return elem.snd;
		}
		return null;
	}
	public boolean put(K k, V v) {
		int start = k.hashCode() % size;
		int index = start;
		remove(k);
		for(int i = 0; i < size; i++) {
			index = (start + i) % size;
			if (contents[index] == null || contents[index] == removed) {
				contents[index] = new Pair<K, V>(k, v);
				return true;
			}
		}
		return false;
	}
	public void remove(K k) {
		int start = k.hashCode() % size;
		for(int i = 0; i < size; i++) {
			int index = (i + start) % size;
			if (contents[index] == null) return;
			if (contents[index] == removed) continue;
			if (contents[index].fst.equals(k)) {
				contents[index] = removed;
				return;
			}
		}
		
	}
 
}
